-- written by Gizmo
----------------------------------------------------------------------
-- If you like Project Phoenix and want to help support the project
-- you could send any donation amount to jessebutler1975@gmail.com at
-- paypal.com
--
-- Thank you for using my product, 
-- I hope you enjoy it as much as I do
----------------------------------------------------------------------
require 'spawner_edit_window'
require 'spawner_tooltip'
require 'spawner_tables'
require 'spawner_hue_picker'

local FASTEST_DELAY_POSSIBLE = 10 --milliseconds
local COMMAND_KEY = "@"
--=====================================================================
--							CORE FUNCTIONS
--=====================================================================
function getIf(mBool, v1, v2)
    if mBool then 
        return v1
    else
        return v2
    end
end

function DoesTemplateExist(filename)
	local check = GetAllTemplateNames()
	for k,v in pairs(check) do
		if(v=="spawner_"..filename) then return true end
	end
	return false
end

function DebugMe(name,message)
	if(this:GetName()==name) then
		print(this:GetName()..":"..message)
	end
end
-----------------------------------------------------------------------
--					HandleChanges(mob,spawn_template)	  	
-----------------------------------------------------------------------
function HandleChanges(mob,spawn_template)
	if(spawn_template == nil) then return end

	local dataTable = StringSplit(spawn_template,COMMAND_KEY)
	for i=1 , #dataTable do
		--first value is always the mob template, rest is code
		if(i > 1) then
			local todo = StringSplit(dataTable[i],":")

			if(todo[1] == "SetHue") then mob:SetHue(tonumber(todo[2])) end
			if(todo[1] == "SetColor") then mob:SetColor(todo[2]) end
			if(todo[1] == "SetScale") then mob:SetScale(Loc(tonumber(todo[2]),tonumber(todo[2]),tonumber(todo[2]))) end
			if(todo[1] == "SetStr") then CallFunctionDelayed(TimeSpan.FromMilliseconds(250),function ( ... ) SetStr(mob,tonumber(todo[2])) end) end
			if(todo[1] == "SetAgi") then CallFunctionDelayed(TimeSpan.FromMilliseconds(250),function ( ... ) SetAgi(mob,tonumber(todo[2])) end) end
			if(todo[1] == "SetInt") then CallFunctionDelayed(TimeSpan.FromMilliseconds(250),function ( ... ) SetInt(mob,tonumber(todo[2])) end) end
			if(todo[1] == "SetCon") then CallFunctionDelayed(TimeSpan.FromMilliseconds(250),function ( ... ) SetCon(mob,tonumber(todo[2])) end) end
			if(todo[1] == "SetWis") then CallFunctionDelayed(TimeSpan.FromMilliseconds(250),function ( ... ) SetWis(mob,tonumber(todo[2])) end) end
			if(todo[1] == "SetWill") then CallFunctionDelayed(TimeSpan.FromMilliseconds(250),function ( ... ) SetWill(mob,tonumber(todo[2])) end) end
			if(todo[1] == "SetName") then 
				mob:SetName(todo[2])
				CallFunctionDelayed(TimeSpan.FromMilliseconds(25),
					function ( ... ) 
						mob:SetName(todo[2]) 
					end) 
			end
			if(todo[1] == "SetTitle") then CallFunctionDelayed(TimeSpan.FromMilliseconds(25),function ( ... ) mob:SetSharedObjectProperty("Title", todo[2]) end) end
			if(todo[1] == "SetTeam") then 
				CallFunctionDelayed(TimeSpan.FromMilliseconds(25), 
				function ( ... ) 
					mob:SetObjVar("MobileTeamType",todo[2])
					mob:SetSharedObjectProperty("Faction", todo[2]) 
				end) 
			end
			if(todo[1] == "SetLocation") then
				if(todo[2] == "SELF") then
					mob:SetWorldPosition(this:GetLoc())
				else
					local coords = StringSplit(todo[2],",")
					mob:SetWorldPosition(Loc(tonumber(coords[1]),tonumber(coords[2]),tonumber(coords[3])))
				end
			end
			if(todo[1] == "SetRotation") then
				local coords = StringSplit(todo[2],",")
				mob:SetRotation(Loc(tonumber(coords[1]),tonumber(coords[2]),tonumber(coords[3])))
			end			
			if(todo[1] == "SetObjVar") then
				local objvardata = StringSplit(todo[2],",")
				mob:SetObjVar(objvardata[1],objvardata[2])
			end
			if(todo[1] == "SetSkill") then
				CallFunctionDelayed(TimeSpan.FromMilliseconds(250),function ( )
					if(mob:HasObjVar("SkillDictionary")) then
					local skilldata = StringSplit(todo[2],",")
					local skillDictionary = mob:GetObjVar("SkillDictionary") or {}
						skillDictionary[skilldata[1].."Skill"].SkillLevel=tonumber(skilldata[2])
						mob:SetObjVar("SkillDictionary",skillDictionary) 
						mob:SendMessage("OnSkillLevelChanged", skilldata[1].."Skill")
					end
				end)
			end
			if(todo[1] == "PlaySimpleEffect") then 
				CallFunctionDelayed(TimeSpan.FromMilliseconds(250),function ( ... ) 
					local effectdata = StringSplit(todo[2],",")
					mob:PlayEffect(effectdata[1],tonumber(effectdata[2])) 
				end) 
			end 
			if(todo[1] == "PlaySound") then CallFunctionDelayed(TimeSpan.FromMilliseconds(250),function ( ... )  mob:PlayObjectSound(todo[2],false) end) end 
			if(todo[1] == "SetAppearanceFromTemplate") then mob:SetAppearanceFromTemplate(todo[2]) end 
			if(todo[1] == "SetInvulnerability") then mob:SetObjVar("Invulnerable",(todo[2])) end
			if(todo[1] == "SetFacing") then mob:SetFacing(todo[2]) end
			if(todo[1] == "SetHealth") then 
				CallFunctionDelayed(TimeSpan.FromMilliseconds(250),
					function ( ... ) 
						mob:SetStatMaxValue("Health", math.floor(tonumber(todo[2]))) 
						SetCurHealth(mob,tonumber(todo[2]))
					end)   
			end
			if(todo[1] == "SetStamina") then 
				CallFunctionDelayed(TimeSpan.FromMilliseconds(250),
					function ( ... ) 
						mob:SetStatMaxValue("Stamina", math.floor(tonumber(todo[2]))) 
						SetCurStamina(mob,tonumber(todo[2]))
					end)   
			end
			if(todo[1] == "SetMana") then 
				CallFunctionDelayed(TimeSpan.FromMilliseconds(250),
					function ( ... ) 
						mob:SetStatMaxValue("Mana", math.floor(tonumber(todo[2]))) 
						SetCurMana(mob,tonumber(todo[2]))
					end)   
			end
			if(todo[1] == "AddModule") then
				if(type(todo[1]) == "string") then
					mob:AddModule(todo[2])
				elseif(type(todo[1])) == "table" then
					local args = StringSplit(todo[2],",")
					for k,v in pairs(args) do
						mob:AddModule(v)
					end
				end
			end
			if(todo[1] == "RemoveModule") then
				if(type(todo[1]) == "string") then
					mob:RemoveModule(todo[2])
				elseif(type(todo[1])) == "table" then
					local args = StringSplit(todo[2],",")
					for k,v in pairs(args) do
						mob:RemoveModule(v)
					end
				end
			end
			if(todo[1] == "SetSharedObjectProperty") then
				local args = StringSplit(todo[2],",")
				mob:SetSharedObjectProperty(arg[1],arg[2])
			end
			if(todo[1] == "SetObjectTag") then mob:SetObjectTag(todo[2]) end								
		end
	end
end

function NextSequence()
	local current_sequence = this:GetObjVar("current_sequence")
	local max_sequence = #this:GetObjVar("templates")
	current_sequence = current_sequence + 1
	if(current_sequence > max_sequence) then
		current_sequence = 1
	end
	this:SetObjVar("current_sequence",current_sequence)
end

function EffectTableContains(effectName)
	local active_effect_table = this:GetObjVar("data.ActiveEffectsList")
	for i, effect in pairs(active_effect_table) do
		if(effect == effectName) then return true end
	end
	return false
end

function EffectCleanup()
	if(this:HasObjVar("data.ActiveEffectsList")) then
		local active_effect_table = this:GetObjVar("data.ActiveEffectsList")
		for i, effect in pairs(active_effect_table) do
			this:StopEffect(effect)
		end		
	end
	this:DelObjVar("data.ActiveEffectsList")
end

function SoundTableContains(soundName)
	local active_sound_table = this:GetObjVar("data.ActiveSoundList")
	for i, sound in pairs(active_sound_table) do
		if(sound == soundName) then return true end
	end
	return false
end

function SoundCleanup()
	if(this:HasObjVar("data.ActiveSoundList")) then
		local active_sound_table = this:GetObjVar("data.ActiveSoundList")
		for i, sound in pairs(active_sound_table) do
			this:StopObjectSound(sound)
		end		
	end
	this:DelObjVar("data.ActiveSoundList")
end

-----------------------------------------------------------------------
--					CheckingCommands(data)		  	
-----------------------------------------------------------------------
function CheckingCommands(data)
	if(data == nil) then return end
	local target = nil
	if(data[1] == "GOTO") then
		this:SetObjVar("current_sequence",tonumber(data[2])-1)
		NextSequence()
		return true
	end
	-----------------------------------------------
	-- OBJECT COMMANDS
	-----------------------------------------------
	if(data[1] == "OBJECT") then
		if(data[2]=="SELF") then
			target = this
		elseif (data[2] == "TRIGMOB") then
			target = this:GetObjVar("View_TriggeredMob") or nil
		elseif (StringSplit(data[2],",")[1]=="TEMPLATE") then
			local matchingObjs = FindObjects(SearchTemplate(StringSplit(data[2],",")[2]))
			if(matchingObjs ~= nil) then
				target = matchingObjs[1]
			else
				local current_sequence = this:GetObjVar("current_sequence")
				current_sequence = current_sequence - 1
				this:SetObjVar("current_sequence",current_sequence)
				NextSequence()
				return true
			end
		else
			target = GameObj(tonumber(data[2]))
		end	
		--==========================
		--		SendMessage()
		--==========================
		if (data[3] == "SendMessage()") then

			local message = data[4]
			if(target ~= nil) then
				if(message == "deactivate" and data[2]=="SELF") then
					ShutDown()
					return true
				end
				target:SendMessage(message)
				if(message=="deactivate") then
					if(data[2] ~= "SELF") then 
						NextSequence()
						return true
					end
				end
			end
			NextSequence()
			return true
		end
		--==========================
		--		SetHue()
		--==========================		
		--version 1.3
		if(data[3] == "SetHue()") then
			local hue = data[4]
			if(target ~= nil) then
				target:SetHue(hue)			
			end
			NextSequence()
			return true
		end
		--==========================
		--		SetColor()
		--==========================		
		if(data[3] == "SetColor()") then
			local color = data[4]
			if(target ~= nil) then
				target:SetColor(color)			
			end
			NextSequence()
			return true
		end
		--==========================
		--		Talk()
		--==========================
		--version 1.4
		if(data[3] == "Talk()") then
			if(data[2]=="SELF") then
				target = this
			else
				target = GameObj(tonumber(data[2]))
			end	

			local message = data[4]
			if(target ~= nil) then
				target:NpcSpeech(message)			
			end
			NextSequence()
			return true
		end
		--==========================
		--		SetAmount()
		--==========================		
        if(data[3] == "SetAmount()") then
            local amount = data[4]
            if(target ~= nil) then
                target:SetObjVar("count_max", tonumber(amount))        
            end
            NextSequence()
            return true
        end
        --==========================
		--		SetScale()
		--==========================
        if(data[3] == "SetScale()") then
            local scale = data[4]
            if(target ~= nil) then
                target:SetScale(Loc(tonumber(scale),tonumber(scale),tonumber(scale)))        
            end
            NextSequence()
            return true
        end
        --==========================
		--		SetDelay() 
		--==========================
        if(data[3] == "SetDelay()") then
            local result = StringSplit(data[4],",")
            local min = result[1]
            local max = result[2]
            if(target ~= nil) then
                target:SetObjVar("min_delay", tonumber(min))        
                target:SetObjVar("max_delay", tonumber(max))        
            end
            NextSequence()
            return true
        end
        --==========================
		--		SetRange() 
		--==========================
        if(data[3] == "SetRange()") then
            local result = StringSplit(data[4],",")
            local min = result[1]
            local max = result[2]
            if(target ~= nil) then
                target:SetObjVar("min_range", tonumber(min))        
                target:SetObjVar("max_range", tonumber(max))        
            end
            NextSequence()
            return true
        end
        --==========================
		--		SetObjVar()
		--==========================
        if(data[3] == "SetObjVar()") then
            local result = StringSplit(data[4],",")
            local objvar = result[1]
            local value = result[2]
            if(target ~= nil) then
            	if(tonumber(value) ~= nil) then
               		target:SetObjVar(objvar, tonumber(value))
               	else
               		target:SetOvjVar(objvar, value)
               	end               
            end
            NextSequence()
            return true
        end
        --==========================
		--		DelObjVar()
		--==========================
        if(data[3] == "DelObjVar()") then
            if(target ~= nil) then
                target:DelObjVar(data[4])               
            end
            NextSequence()
            return true
        end 
        --==========================
		--		RotateBy()
		--==========================
        if(data[3] == "RotateBy()") then
            local result = StringSplit(data[4],",")
            local x = result[1]
            local y = result[2]
            local z = result[3]
            local orig_rotation = target:GetRotation()
            if(target ~= nil) then
                target:SetRotation(Loc(orig_rotation.X + tonumber(x), orig_rotation.Y + tonumber(y), orig_rotation.Z + tonumber(z)))               
            end
            NextSequence()
            return true
        end                
        --==========================
		--		SetLocation()
		--==========================
        if(data[3] == "SetLocation()") then
            local result = StringSplit(data[4],",")
            local x = result[1]
            local y = result[2]
            local z = result[3]
            if(target ~= nil) then
                target:SetWorldPosition(Loc(tonumber(x),tonumber(y),tonumber(z)))               
            end
            NextSequence()
            return true
        end		
		--==========================
		--		PlayEffect()
		--==========================
        if(data[3] == "PlayEffect()") then
        	local effectdata = StringSplit(data[4],",")
            if(target ~= nil) then
            	target:PlayEffect(effectdata[1],tonumber(effectdata[2]))        
            end
            if not(this:HasObjVar("data.ActiveEffectsList")) then
            	local data = {}
            	table.insert(data,effect)
            	this:SetObjVar("data.ActiveEffectsList",data)
            else
            	if not(EffectTableContains(effect)) then
            		local data = this:GetObjVar("data.ActiveEffectsList")
            		table.insert(data,effect)
            		this:SetObjVar("data.ActiveEffectsList",data)
            	end
            end
            NextSequence()
            return true
        end
		--==========================
		--		StopEffect()
		--==========================
        if(data[3] == "StopEffect()") then
            local effect = data[4]
            if(target ~= nil) then
            	target:StopEffect(effect)        
            end
            NextSequence()
            return true
        end 
		--==========================
		--		PlaySound()
		--==========================
        if(data[3] == "PlaySound()") then
            local sound = data[4]
            if(target ~= nil) then
            	target:PlayObjectSound(sound)        
            end
            if not(this:HasObjVar("data.ActiveSoundList")) then
            	local data = {}
            	table.insert(data,sound)
            	this:SetObjVar("data.ActiveSoundList",data)
            else
            	if not(SoundTableContains(sound)) then
            		local data = this:GetObjVar("data.ActiveSoundList")
            		table.insert(data,sound)
            		this:SetObjVar("data.ActiveSoundList",data)
            	end
            end

            NextSequence()
            return true
        end        
        return false       
	end
	---------------------------------------------------------------------------------------
	--								OBJECT LIST FUNCTIONS
	---------------------------------------------------------------------------------------
	if(data[1] == "OBJECTLIST") then
		local activate_list = StringSplit(data[3],",")
		--==========================
		--		SendMessage()
		--==========================
		--version 1.4	
		if (data[4] == "SendMessage()") then
			local message = data[5]
			if(data[2]=="RANDOM") then
				target = GameObj(tonumber(activate_list[math.random(1,#activate_list)]))
				if(target ~= nil) then
					target:SendMessage(message)
				end
			end
			if(data[2]=="ALL") then
				for index = 1, #activate_list do
					target = GameObj(tonumber(activate_list[index]))
					if(target ~= nil) then
						target:SendMessage(message)
					end
				end
			end
			NextSequence()
			return true
		end
		--==========================
		--		SetHue()
		--==========================		
		if(data[4] == "SetHue()") then
			local hue = data[5]
			if(data[2]=="RANDOM") then
				target = GameObj(tonumber(activate_list[math.random(1,#activate_list)]))
				if(target ~= nil) then
					target:SetHue(hue)
				end
			end
			if(data[2]=="ALL") then
				for index = 1, #activate_list do
					target = GameObj(tonumber(activate_list[index]))
					if(target ~= nil) then
						target:SetHue(hue)
					end
				end
			end
			NextSequence()
			return true
		end
		if(data[4] == "SetColor()") then
			local color = data[5]
			if(data[2]=="RANDOM") then
				target = GameObj(tostring(activate_list[math.random(1,#activate_list)]))
				if(target ~= nil) then
					target:SetColor(color)
				end
			end
			if(data[2]=="ALL") then
				for index = 1, #activate_list do
					target = GameObj(tostring(activate_list[index]))
					if(target ~= nil) then
						target:SetColor(color)
					end
				end
			end
			NextSequence()
			return true
		end
	end
	---------------------------------------------------------------------------------------
	--								WAITFOR FUNCTIONALITY
	---------------------------------------------------------------------------------------
	if(data[1] == "WAITFOR") then
		if(data[3]=="SELF") then
			target = this
		else
			if(data[3] ~= nil) then target = GameObj(tonumber(data[3])) end
		end
		--oh its a wait for trigger, so set target to me please
		if(data[2] =="TRIGGER") then
			target = this
		end	
		if(data[2]=="OBJECT") then
			if(target ~= nil) then
				---------------------------------
				--  COMPAIR VS OBJVAR OF OBJECT
				---------------------------------
				--WAITFOR@OBJECT@<objectid>@GetObjVar()@<objvarname>@<condition>@<value>
				if(data[4] == "GetObjVar()") then
					local comp1 = target:GetObjVar(data[5])
					local comp2 = nil
					if(type(comp1) == "string") then
						comp2 = tostring(data[7])
					elseif(type(comp1) == "boolean") then
						if(data[7] == "true") then comp2 = true end
						if(data[7] == "false") then comp2 = false end
					elseif(type(comp1) == "number") then
						comp2 = tonumber(data[7])
					end
					if(comp1 == nil or comp2 == nil) then return false end
					if(data[6] == ">") then
						if(target:GetObjVar(data[5]) > tonumber(data[7])) then 
							NextSequence()
							return true
						end
					end
					if(data[6] == "<") then
						if(target:GetObjVar(data[5]) < tonumber(data[7])) then 
							NextSequence()
							return true
						end
					end
					if(data[6] == "=") then

						if(comp1 == comp2) then
							NextSequence()
							return true
						end
					end
					if(data[6] == ">=") then
						if(target:GetObjVar(data[5]) >= tonumber(data[7])) then 
							NextSequence()
							return true
						end
					end
					if(data[6] == "<=") then
						if(target:GetObjVar(data[5]) <= tonumber(data[7])) then 
							NextSequence()
							return true
						end
					end
					if(data[6] == "~=") then
						if(target:GetObjVar(data[5]) ~= tonumber(data[7])) then 
							NextSequence()
							return true
						end
					end																									
				end
				if(data[4] == "HasObjVar()") then
					if(target:HasObjVar(data[5])) then
						NextSequence()
						return true
					end
				end
				if(data[4] == "HasModule()") then
					if(target:HasModule(data[5])) then
						NextSequence()
						return true
					end
				end
			end
	end
	
	---------------------------------------------------------------------------------------
	--								TRIGGER FUNCTIONS
	---------------------------------------------------------------------------------------		
	if(data[2]=="TRIGGER") then
		if(this:GetObjVar("View_Triggered")) then
			NextSequence()
			this:SetObjVar("View_Triggered",false)
			return true
		end
	end
	end
	if(data[1]=="BROADCAST") then
		local message = data[2]
		local onlineUsers = GlobalVarRead("User.Online")
		for user,y in pairs(onlineUsers) do
			user:SystemMessage(message,"event")
			user:SystemMessage(message)
		end
		NextSequence()
		return true
		-- if(args[2] == "true") then
		-- 	ServerBroadcast(args[1],true)
		-- 	NextSequence()
		-- 	return true
		-- else
		-- 	ServerBroadcast(args[1],false)
		-- 	NextSequence()
		-- 	return true
		-- end
	end
	if(data[1]=="DESTROYSELF") then
		if(this:HasObjVar("parentSpawner")) then
			GameObj(this:GetObjVar("parentSpawner")):SendMessage("mobHasDied",this)
		end
		for k,v in pairs(this:GetObjVar("mob_list")) do
			v:Destroy()
		end
		this:Destroy()
		return true	
	end
end

-----------------------------------------------------------------------
--			 			SequentialSpawnObject()			  	
-----------------------------------------------------------------------
--this is the spawn cycle for the sequential spawn system
function SequentialSpawnObject()
	--this is a fix for when spawners boot up, they should never be at sequence 0
	if(this:GetObjVar("current_sequence") == 0) then this:SetObjVar("current_sequence",1) end
	local sleeping = this:GetObjVar("sleeping")
	--are we sleeping?
	if not(sleeping) then

		if(this:HasObjVar("PhaseSpawned")) then return end

		local templates = this:GetObjVar("templates")
		local min_range = this:GetObjVar("min_range")
		local max_range = this:GetObjVar("max_range")
		local count_max = this:GetObjVar("count_max")
		local spawn_mobile = ""
		local spawn_template = templates[this:GetObjVar("current_sequence")]

		local data = StringSplit(spawn_template,COMMAND_KEY)

		if(data == nil) then return end

		if(CheckingCommands(data) == true) then
			UpdateToolTip(this,false) 
			return true 
		end

		local spawn_mobile = data[1]
		local amount_to_spawn = this:GetObjVar("count_max")

		for i=1, amount_to_spawn do
			CreateObj(spawn_mobile,this:GetLoc():Project(math.random(0,360), math.random(min_range,max_range)),"mobileSpawned",1,spawn_template)						
		end
		this:SetObjVar("PhaseSpawned",true)
	end
end

-----------------------------------------------------------------------
--						  	SpawnObject
-----------------------------------------------------------------------
--this will spawn an mobile from the templates list
function SpawnObject()
	if(this:GetObjVar("templates") ~= nil) then
		local templates = this:GetObjVar("templates")
		local min_range = this:GetObjVar("min_range")
		local max_range = this:GetObjVar("max_range")
		local count_max = this:GetObjVar("count_max")
		local spawn_mobile = ""
		local spawn_template = templates[math.random(1,#templates)]
		local data = StringSplit(spawn_template,COMMAND_KEY)
		local spawn_mobile = data[1]
		--version 1.3
		if(CheckingCommands(data) == true) then
			UpdateToolTip(this,false)
			return nil
		end
		--repopulate the spawner virtually
		if(this:GetObjVar("sleeping") == true) then
			local reset_list = this:GetObjVar("reset_list") or {}
			if(reset_list ~= nil and (#reset_list < count_max)) then
				table.insert(reset_list,spawn_mobile)
				this:SetObjVar("reset_list",reset_list)

				local current_count = this:GetObjVar("count_current")
				current_count = current_count + 1
				this:SetObjVar("count_current",current_count)				
				UpdateToolTip(this,false)
			end
			return
		end

		if(type(templates) == "table") then
			CreateObj(spawn_mobile,this:GetLoc():Project(math.random(0,360), math.random(min_range,max_range)),"mobileSpawned",1,spawn_template)
		end
	end
end

function GetBoundries()
	local location = this:GetLoc()
	local range = this:GetObjVar("View_Range")
	local topLeft = Loc2(location.X-range,location.Z-range)
	local topRight = Loc2(location.X+range,location.Z-range)
	local bottomRight = Loc2(location.X+range,location.Z+range)
	local bottomLeft = Loc2(location.X-range,location.Z+range)
	return Box2(topLeft,topRight,bottomLeft,bottomRight)		
end

-----------------------------------------------------------------------
--						  On Module Attached
-----------------------------------------------------------------------
--when this module is attached this will set up various values for
--the spawner system
function OnModuleAttached()
	--set up the basic variables
	local templates = {}
	this:SetObjVar("templates",templates)
	--this:SetObjVar("active",true)
	this:SetObjVar("mob_list",{})
	this:SetObjVar("config",ConfigurationData)
	this:SetObjVar("loot",LootData)
	this:SetObjVar("reset_list",{})
	this:SetObjVar("View_OnEnterMessage","")
	this:SetObjVar("View_OnExitMessage","")
	this:SetObjVar("View_RegionName","Custom Spawner")
	this:SetObjVar("View_TriggeredMob",0)
	this:SetObjVar("View_Range",5)
	this:SetObjVar("View_ReTriggerDelay",30)
	this:SetObjVar("View_NextTrigTime",os.time()+30)
	this:SetObjVar("View_Triggered",false)
	this:SetObjVar("View_TriggerActive",false)
	if(initializer ~= nil) then
		if(initializer.Loot ~= nil) then
			local lootList = {}
			--local loot_table = {data[1],data[2],data[3]}
			for i, lootItem in pairs(initializer.Loot) do
				local loot = {lootItem["Template"],lootItem["Chance"],lootItem["Amount"]}
				table.insert(lootList,loot)
			end
			this:SetObjVar("loot",lootList)
		end
		if(initializer.Templates ~= nil) then
			local templates = {}
			for i, template in pairs(initializer.Templates) do
				table.insert(templates,template)
			end
			this:SetObjVar("templates",templates)
		end
		--initialize config
		local min_delay = this:GetObjVar("min_delay")
		local max_delay = this:GetObjVar("max_delay")
		local min_range = this:GetObjVar("min_range")
		local max_range = this:GetObjVar("max_range")
		local count_max = this:GetObjVar("count_max")
		local spawner_tag = this:GetObjVar("spawner_tag")

		local data =
		{
			{"min_delay", min_delay},
			{"max_delay", max_delay},
			{"min_range", min_range},
			{"max_range", max_range},
			{"count_max", count_max},
			{"spawner_tag", spawner_tag},
		}
		this:SetObjVar("config",data)
		this:SetObjVar("current_sequence",0)
		this:SetObjVar("kill_count",0)	
		this:SetObjVar("View_Range",5)
		AddView("SpawnerView", SearchRect(GetBoundries()))
	end

	AddUseCase(this,"Hue",false)
	AddUseCase(this,"Edit",true)
	AddUseCase(this,"Move",false)
	AddUseCase(this,"Duplicate",false)
	AddUseCase(this,"Show Bounds",false)
	AddUseCase(this,"Remove Bounds",false)
	AddUseCase(this,"Adjust View",false)

	--like a dumb dumb I made this a string? but people are
	--using this system so their old spawers eh, ill just
	--hack a fix here cause im lazy on this issue
	--this is a edit to fix stuff not spawning when a spawner
	--is spawned
	if(this:GetObjVar("active")=="true") then
		this:SetObjVar("active",true)
	else
		this:SetObjVar("active",false)
	end

	this:ScheduleTimerDelay(TimeSpan.FromMilliseconds(GetRespawnDelay()), "spawnerTimer")
	
	UpdateToolTip(this,false)	
end

-----------------------------------------------------------------------
--						  	Update Config
-----------------------------------------------------------------------
--when the update button is clicked on the Config tab it will update
--all the values on the spawners objvars.
function UpdateConfig()
	local config = 
	{
		{"min_delay", this:GetObjVar("min_delay")},
		{"max_delay", this:GetObjVar("max_delay")},
		{"min_range", this:GetObjVar("min_range")},
		{"max_range", this:GetObjVar("max_range")},
		{"count_max", this:GetObjVar("count_max")},
		{"spawner_tag", this:GetObjVar("spawner_tag")},
	}
	this:SetObjVar("config",config)
end

-----------------------------------------------------------------------
--						    Despawn Object
-----------------------------------------------------------------------
--when this is called it will despawn everything that is currently
--active from this spawner
function ShutDown()
	this:SetObjVar("active",false)
	local mob_list = this:GetObjVar("mob_list")
	if(#mob_list > 0) then
		for i=#mob_list, 1, -1 do
			local object = GameObj(tonumber(mob_list[i]))
			if(object:HasObjVar("parentSpawner") and object:HasModule("spawner")) then
				local spwn_mob_list = object:GetObjVar("mob_list")
				if(#spwn_mob_list > 0) then
					for d=#spwn_mob_list, 1, -1 do
						local spwn_object = GameObj(tonumber(spwn_mob_list[d]))
						spwn_object:Destroy()
					end
				end
			end
			object:Destroy()
		end
	end
	this:SetObjVar("mob_list",{})
	this:SetObjVar("reset_list",{})
	this:SetObjVar("count_current", 0)
	this:DelObjVar("PhaseSpawned")
 	this:SetObjVar("kill_count",0)
 	this:SetObjVar("current_sequence",1)
	if(this:HasTimer("spawnerTimer")) then this:RemoveTimer("spawnerTimer") end
	EffectCleanup()
	SoundCleanup()
end

-----------------------------------------------------------------------
--						  Get Respawn Delay
-----------------------------------------------------------------------
--this sets the respawn delay for the next mobile respawn time. this 
--value is determined from a math.random(min_delay,max_delay)
function GetRespawnDelay()
	local min_delay = this:GetObjVar("min_delay")
	local max_delay = this:GetObjVar("max_delay")
	if(max_delay < min_delay) then
		ShutDown()
	end
	min_delay = min_delay * 1000
	max_delay = max_delay * 1000
	local time_delay = math.random(min_delay,max_delay) or FASTEST_DELAY_POSSIBLE
	if(time_delay <= 0) then time_delay = FASTEST_DELAY_POSSIBLE end
	return time_delay
end

-----------------------------------------------------------------------
--					    Create Loot On Mobile
-----------------------------------------------------------------------
function CreateLootOnMobile(object)
	local backpackObj = object:GetEquippedObject("Backpack")
	if(object:HasModule("container")) then 
		backpackObj = object 
	end
	if(#this:GetObjVar("loot") ~= 0) then
		local lootTemplate = ""
		if(this:GetObjVar("loot_gen_type") == "random") then
			lootTemplate = this:GetObjVar("loot")[math.random(1,#this:GetObjVar("loot"))]	
			local itemTemplate = lootTemplate[1]
			local itemChance = lootTemplate[2]
			local itemQuantity = lootTemplate[3]
			local data = StringSplit(itemTemplate,"@")
			local params = {}
			if(type(data) == "table") then
				itemTemplate = data[1]
				if(itemTemplate == "RandomWeapon") then
					itemTemplate = LootTable.Functions.Get.RandomWeapon()
				elseif(itemTemplate == "RandomArmor") then
					itemTemplate = LootTable.Functions.Get.RandomArmor()
				elseif(itemTemplate == "RandomOffHand") then
					itemTemplate = LootTable.Functions.Get.RandomShield()
				elseif(itemTemplate == "RandomCoins") then
					itemTemplate = "coin_purse"
					local values = StringSplit(data[2],",")
					itemQuantity = math.random(tonumber(values[1]),tonumber(values[2]))
				end
				params = {Rank=data[2]}
			end
			if(tonumber(itemQuantity) > 1) then
				if(math.random(1,itemChance)==1) then
					CreateObjInContainer(itemTemplate, backpackObj, GetRandomDropPosition(backpackObj), "createdStackableLootOnMobile", tonumber(itemQuantity))
				end
			else
				if(math.random(1,itemChance)==1) then
					CreateObjInContainer(itemTemplate, backpackObj, GetRandomDropPosition(backpackObj), "createdLootOnMobile",1,params)
				end
			end
		elseif(this:GetObjVar("loot_gen_type") == "cascade") then
			for k,v in pairs(this:GetObjVar("loot")) do	
				local itemTemplate = v[1]
				local itemChance = v[2]
				local itemQuantity = v[3]
				local data = StringSplit(itemTemplate,"@")
				local params = {}
				if(type(data) == "table") then
					itemTemplate = data[1]
					if(itemTemplate == "RandomWeapon") then
						itemTemplate = LootTable.Functions.Get.RandomWeapon()
					elseif(itemTemplate == "RandomArmor") then
						itemTemplate = LootTable.Functions.Get.RandomArmor()
					elseif(itemTemplate == "RandomOffHand") then
						itemTemplate = LootTable.Functions.Get.RandomShield()
					elseif(itemTemplate == "RandomCoins") then
						itemTemplate = "coin_purse"
						local values = StringSplit(data[2],",")
						itemQuantity = math.random(tonumber(values[1]),tonumber(values[2]))
					end
					params = {Rank=data[2]}
				end								
				if(tonumber(itemQuantity) > 1) then
					if(math.random(1,itemChance)==1) then
						CreateObjInContainer(itemTemplate, backpackObj, GetRandomDropPosition(backpackObj), "createdStackableLootOnMobile", tonumber(itemQuantity))
					end
				else
					if(math.random(1,itemChance)==1) then
						CreateObjInContainer(itemTemplate, backpackObj, GetRandomDropPosition(backpackObj), "createdLootOnMobile",1,params)
					end
				end			
			end			
		end
	end
end

-----------------------------------------------------------------------
--					    Spawner Update
-----------------------------------------------------------------------
--this controls updated items that need to be active when the spawner
--code kicks in for the first time on older spawners. 
function SpawnerUpdate()
	
	if not(this:HasObjVar("kill_count")) then
		this:SetObjVar("kill_count",0)
	end
	--this keeps track of the current sequence of the
	--sequential spawning system. the sequence will not be
	--updated unless the total_kill_count has been reached
	if not(this:HasObjVar("current_sequence")) then
		this:SetObjVar("current_sequence",1)
	end	

	EffectCleanup()
	SoundCleanup()

	--make sure the timer starts correctly
	if(this:GetObjVar("active") == true) then
		this:ScheduleTimerDelay(TimeSpan.FromMilliseconds(GetRespawnDelay()), "spawnerTimer")
	else
		this:RemoveTimer("spawnerTimer")
	end
	--for legacy spawners
	if not(this:HasObjVar("loot_gen_type")) then
		this:SetObjVar("loot_gen_type","random")
	end

	if(ProjectPhoenix ~= nil) then
		ProjectPhoenix.ApplyVersionInfo(this)
	end
end

RegisterEventHandler(EventType.EnterView, "SpawnerView",
	function ( mobileObj )
		if not(this:GetObjVar("active")) then return end
    	if (mobileObj:IsPlayer()) then
    		if not(this:GetObjVar("View_TriggerActive")) then
    			--must do on all player mobiles
    			if(this:GetObjVar("active")) then
    				mobileObj:SystemMessage(this:GetObjVar("View_OnEnterMessage"),"info")
    				mobileObj:SetObjVar("CustomRegion",this:GetObjVar("View_RegionName"))
    			end
    		else
	    		--optional checks
	    		--can we trigger?
	    		if(os.time() > this:GetObjVar("View_NextTrigTime")) then
	    			mobileObj:SystemMessage(this:GetObjVar("View_OnEnterMessage"),"info")
	    			--set the next delay time time.now + retrigdelay objvar
	    			this:SetObjVar("View_NextTrigTime",os.time()+this:GetObjVar("View_ReTriggerDelay"))
	    			this:SetObjVar("View_TriggeredMob",mobileObj.Id)
	    			this:SetObjVar("View_Triggered",true)
	    			mobileObj:SetObjVar("CustomRegion",this:GetObjVar("View_RegionName"))
	    			if(IsGod(mobileObj)) then
	    				mobileObj:SystemMessage("Triggered at: [00AA00]" .. os.clock() .. " [FFFFFF]next retrigger in [00AA00]"..this:GetObjVar("View_ReTriggerDelay") .. " [FFFFFF]seconds.")
	    			end
	    		end
    		end	 		
    	end
	end)

RegisterEventHandler(EventType.LeaveView, "SpawnerView", 
    function (mobileObj)
    	if (mobileObj:IsPlayer()) then
    		if(this:GetObjVar("active")) then
    			mobileObj:SystemMessage(this:GetObjVar("View_OnExitMessage"),"info")
    			mobileObj:DelObjVar("CustomRegion")
    		end
    	end
    end)

function CreateTemplate(fullpath,user)
	local file = io.open(fullpath,"w")

	file:write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n")
	file:write("<ObjectTemplate xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">\n")
	file:write("\t<ClientId>2</ClientId>\n")
	file:write("\t<Color>" .. this:GetColor() .. "</Color>\n")
	file:write("\t<Hue>" .. this:GetHue() .. "</Hue>\n")
	file:write("\t<Name>" .. this:GetName() .. "</Name>\n")
	
	file:write("\t<ObjectVariableComponent>\n")
		file:write("\t\t<StringVariable Name=\"SearchType\">Spawner</StringVariable>\n")
		file:write("\t\t<StringVariable Name=\"Gizmos_Search_Function\">Don't Change This</StringVariable>\n")
		file:write("\t\t<StringVariable Name=\"templates\">empty</StringVariable>\n")
		file:write("\t\t<DoubleVariable Name=\"min_delay\">" .. this:GetObjVar("min_delay") .. "</DoubleVariable>\n")
		file:write("\t\t<DoubleVariable Name=\"max_delay\">" .. this:GetObjVar("max_delay") .. "</DoubleVariable>\n")
		file:write("\t\t<DoubleVariable Name=\"min_range\">" .. this:GetObjVar("min_range") .. "</DoubleVariable>\n")
		file:write("\t\t<DoubleVariable Name=\"max_range\">" .. this:GetObjVar("max_range") .. "</DoubleVariable>\n")
		file:write("\t\t<DoubleVariable Name=\"count_current\">0</DoubleVariable>\n")
		file:write("\t\t<DoubleVariable Name=\"count_max\">" .. this:GetObjVar("count_max") .. "</DoubleVariable>\n")
		file:write("\t\t<StringVariable Name=\"NoReset\">true</StringVariable>\n")
		file:write("\t\t<BoolVariable Name=\"WorldSave\">true</BoolVariable>\n")
		if(this:GetObjVar("active")==true) then
			file:write("\t\t<StringVariable Name=\"active\">true</StringVariable>\n")
		else
			file:write("\t\t<StringVariable Name=\"active\">false</StringVariable>\n")		
		end
		file:write("\t\t<StringVariable Name=\"spawner_tag\">" .. this:GetObjVar("spawner_tag") .. "</StringVariable>\n")
		file:write("\t\t<StringVariable Name=\"mob_list\">empty</StringVariable>\n")
		file:write("\t\t<StringVariable Name=\"config\">empty</StringVariable>\n")
		file:write("\t\t<StringVariable Name=\"loot\">empty</StringVariable>\n")
		file:write("\t\t<StringVariable Name=\"current_sequence\">1</StringVariable>\n")
		file:write("\t\t<StringVariable Name=\"kill_count\">0</StringVariable>\n")
		if(this:GetObjVar("sequential")) then 
			file:write("\t\t<BoolVariable Name=\"sequential\">true</BoolVariable>\n") 
		else
			file:write("\t\t<BoolVariable Name=\"sequential\">false</BoolVariable>\n")
		end	 
		file:write("\t\t<BoolVariable Name=\"sleeping\">false</BoolVariable>\n")
		if(this:GetObjVar("loot_gen_type")=="random") then
			file:write("\t\t<StringVariable Name=\"loot_gen_type\">random</StringVariable>\n")
		end
		if(this:GetObjVar("loot_gen_type")=="cascade") then
			file:write("\t\t<StringVariable Name=\"loot_gen_type\">cascade</StringVariable>\n")		
		end

	file:write("\t</ObjectVariableComponent>\n")
	
	file:write("\t<ScriptEngineComponent>\n")
		file:write("\t\t<LuaModule Name=\"spawner\">\n")
			file:write("\t\t\t<Initializer>\n")
			file:write("\t\t\t{\n")
				WriteLootItems(file)
				WriteTemplateList(file)
	        file:write("\t\t\t}\n")
			file:write("\t\t\t</Initializer>\n")
		file:write("\t\t</LuaModule>\n")
	file:write("\t</ScriptEngineComponent>\n")
	file:write("</ObjectTemplate>")

	file:close()
	user:SystemMessage("[FF7700]"..fullpath.."[FFFFBB]... scripted and saved!")		
end

function tablelength(table)
	local count = 0
	for _ in pairs(table) do count = count + 1 end
	return count
end

function WriteLootItems(file)
	if(tablelength(this:GetObjVar("loot")) <= 0) then return end
	file:write("\t\t\t\tLoot =\n")
	file:write("\t\t\t\t{\n")
	for i,lootlist in pairs(this:GetObjVar("loot")) do
		file:write("\t\t\t\t\t{\n")
		file:write("\t\t\t\t\t\tTemplate=\"" .. lootlist[1] .. "\",\n")
		file:write("\t\t\t\t\t\tChance=\"" .. lootlist[2] .. "\",\n")
		file:write("\t\t\t\t\t\tAmount=\"" .. lootlist[3] .. "\",\n")
		file:write("\t\t\t\t\t},\n")
	end
	file:write("\t\t\t\t},\n")
end

function WriteTemplateList(file)
	if(tablelength(this:GetObjVar("templates")) <= 0) then return end
	file:write("\t\t\t\tTemplates =\n")
	file:write("\t\t\t\t{\n")
	for i,template in pairs(this:GetObjVar("templates")) do
		file:write("\t\t\t\t\t\""..template.."\",\n")
	end
	file:write("\t\t\t\t},\n")
end

--======================================================================
--							EVENT HANDLERS
--======================================================================
----------------------------------------------------------
--EVENTTYPE - ModuleAttached
----------------------------------------------------------
--fire up OnModuleAttached, should only be firing when the 
--gizmo_simple_spawner object is spawned. Though this could be
--attached to about anything and probably run okay. 
RegisterSingleEventHandler(EventType.ModuleAttached, GetCurrentModule(), 
	function()
		OnModuleAttached()
		
	end)

----------------------------------------------------------
--EVENTTYPE - CreatedObject - "created_region_bound_marker"
----------------------------------------------------------
RegisterEventHandler(EventType.CreatedObject, "created_region_bound_marker", 
	function(success, objref, index)
		local markers = this:GetObjVar("bounds_objects")
		if(markers == nil) then
			markers = {}
		end
		if(success) then
			table.insert(markers,objref)
			this:SetObjVar("bounds_objects",markers)
		end
	end)

----------------------------------------------------------
-- EVENTTYPE - Message - "UseObject"
----------------------------------------------------------
-- when you click the spawner in game, it will open up the
-- edit menu for you. you can also can use another option
-- by right clicking the spawner and selecting "Edit"
RegisterEventHandler(EventType.Message, "UseObject", 
    function(user,usedType)
        if(usedType == "Hue") then
        	user:CloseDynamicWindow("EditHueDialog")
        	SimpleSpawnerHuePicker(user,this)
        end    	
        if(usedType == "Edit") then 
        	user:CloseDynamicWindow("EditSpawnerDialog")
        	SimpleSpawnerEditWindow(user,this,"Config")
        	UpdateToolTip(this,true) 
        end
        if(usedType == "Move") then
        	user:RequestClientTargetLoc(this,"moveToTarget")
        	user:SystemMessage("select a location to move this spawner too")
        end
        if(usedType == "Duplicate") then
        	user:RequestClientTargetLoc(this,"duplicateAtTarget")
        	user:SystemMessage("select a location you wish to place a copy of this spawner at")
        end
        if(usedType == "Show Bounds" and this:GetObjVar("bounds_objects") == nil) then
        	this:SetObjVar("bounds_objects",{})
        	local location = this:GetLoc()
        	local range = this:GetObjVar("View_Range")
        	local topLeft = Loc2(location.X-range,location.Z-range)
			local topRight = Loc2(location.X+range,location.Z-range)
			local bottomRight = Loc2(location.X+range,location.Z+range)
			local bottomLeft = Loc2(location.X-range,location.Z+range)
			for x = topLeft.X, topRight.X do
				local loc = Loc(x,0,topLeft.Z)
				CreateObj("spawner_region_bound_marker",loc,"created_region_bound_marker")
			end
			for x = topLeft.X, topRight.X do
				local loc = Loc(x,0,bottomLeft.Z)
				CreateObj("spawner_region_bound_marker",loc,"created_region_bound_marker")
			end
			for z = topLeft.Z+1, bottomLeft.Z-1 do
				local loc = Loc(topLeft.X,0,z)
				CreateObj("spawner_region_bound_marker",loc,"created_region_bound_marker")
			end
			for z = topRight.Z+1, bottomRight.Z-1 do
				local loc = Loc(topRight.X,0,z)
				CreateObj("spawner_region_bound_marker",loc,"created_region_bound_marker")
			end
        end
        if(usedType == "Remove Bounds") then
			local markers = this:GetObjVar("bounds_objects")
			for i, marker in pairs(markers) do
				marker:Destroy()
			end
			this:DelObjVar("bounds_objects")
        end
        if(usedType == "Adjust View") then
			DelView("SpawnerView")
			AddView("SpawnerView", SearchRect(GetBoundries()))        	
        end               
    end)

----------------------------------------------------------
-- EVENTTYPE - ClientTargetLocResponse - "moveToTarget"
----------------------------------------------------------
--on the context menu when you choose Move this will be fired
--and move the spawner to the selected location
RegisterEventHandler(EventType.ClientTargetLocResponse,"moveToTarget",
	function(success,location)
		if(success and location ~= nil) then
			this:PlayEffect("TeleportToEffect")
			this:SetWorldPosition(location)
			this:PlayEffect("TeleportFromEffect")
		end
	end)

----------------------------------------------------------
-- EVENTTYPE - ClientTargetLocResponse - "duplicateAtTarget"
----------------------------------------------------------
--on the context menu when you choose Move this will be fired
--and move the spawner to the selected location
RegisterEventHandler(EventType.ClientTargetLocResponse,"duplicateAtTarget",
	function(success,location)
		if(success and location ~= nil) then
			CreateObj("spawner",location,"duplicateCreated")
		end
	end)

----------------------------------------------------------
--EVENTTYPE - CreatedObject - "duplicateCreated"
----------------------------------------------------------
--when you create a duplicate of the spawner this will
--copy all the data over to the new spawner
--there is a 1 second delay before the data gets put on
--the duplicated spawner
RegisterEventHandler(EventType.CreatedObject, "duplicateCreated", 
	function(success, objref, index)
		if( success) then
			CallFunctionDelayed(TimeSpan.FromSeconds(1),function ( ... )
				objref:SetColor(this:GetColor())
				objref:SetName(this:GetName())
				objref:SetObjVar("templates",this:GetObjVar("templates"))
				objref:SetObjVar("min_delay",this:GetObjVar("min_delay"))
				objref:SetObjVar("max_delay",this:GetObjVar("max_delay"))
				objref:SetObjVar("min_range",this:GetObjVar("min_range"))
				objref:SetObjVar("max_range",this:GetObjVar("max_range"))
				objref:SetObjVar("count_max",this:GetObjVar("count_max"))
				objref:SetObjVar("spawner_tag",this:GetObjVar("spawner_tag"))
				objref:SetObjVar("config",this:GetObjVar("config"))
				objref:SetObjVar("loot",this:GetObjVar("loot"))
				objref:SetObjVar("current_sequence",0)
				objref:SetObjVar("kill_count",0)
				objref:SetObjVar("sequential",this:GetObjVar("sequential"))
				UpdateToolTip(objref,false)
			end)
		end
	end)

----------------------------------------------------------
--EVENTTYPE - Message - "mobHasDied"
----------------------------------------------------------
--this message is sent from the base_mobile.lua file
--You can put the following into the DoMobileDeath(damager) function
--
--	if(this:HasObjVar("parentSpawner")) then
--		this:SendMessage("mobHasDied", damager,this)
--	end
--
-- you could place this near any of the SendMessage calls and it should
-- be fine.

--version 1.3
--this event handler took a huge overhaul in how it works
--it corrected some bugs and timing issues
RegisterEventHandler(EventType.Message, "mobHasDied", 
    function(deadmob)
    	if(type(deadmob)=="number") then deadmob = GameObj(deadmob) end
    	if(this:GetObjVar("active")) then
			local mob_list = this:GetObjVar("mob_list")
			--fix for objects still triggering the spawners death cycle hehe.
			deadmob:DelObjVar("parentSpawner")
	    	if(this:GetObjVar("sequential")) then
 				local kill_count = this:GetObjVar("kill_count")
 				kill_count = kill_count + 1
 				if(deadmob:HasObjVar("is_spawner_boss")) then kill_count = this:GetObjVar("count_max") end
 				if(kill_count >= this:GetObjVar("count_max")) then
 					local current_sequence = this:GetObjVar("current_sequence")
 					this:DelObjVar("PhaseSpawned")
 					current_sequence = current_sequence + 1
 					if(current_sequence > #this:GetObjVar("templates")) then
 						current_sequence = 1
 						kill_count = 0
 					else
 						kill_count = 0
 					end
 					this:SetObjVar("count_current",0)
 					this:SetObjVar("current_sequence",current_sequence)
 				else
 					--for debug later
 				end
 				this:SetObjVar("kill_count",kill_count)
    		else
	    		local current_count = this:GetObjVar("count_current")
	    		local current_count = current_count - 1

	    		-- make sure current_count isn't less then zero
	    		if(current_count <= 0) then current_count = 0 end	
	    		this:SetObjVar("count_current",current_count)
	    	end	
    		--remove the mob from the mob_list objvar
    		for i=1, #mob_list do
    			if(mob_list[i] == deadmob.Id) then
    				table.remove(mob_list,i)
    				this:SetObjVar("mob_list",mob_list)
    			end
    		end
    		--create loot if needed

    		if(deadmob:IsMobile()) then
    			CreateLootOnMobile(deadmob) 
    		end
    		UpdateToolTip(this,false)
    	end
    end)

----------------------------------------------------------
--EVENTTYPE - Message - "Sleep"
----------------------------------------------------------
--this handles the sleep message
RegisterEventHandler(EventType.Message, "Sleep", 
    function()
    	if(this:GetObjVar("sequential")==true) then
    		print("I dont sleep during sequential spawns sorry.")
    		return
    	end
    	if not(this:GetObjVar("sleeping")) then
    	local mob_list = this:GetObjVar("mob_list")
    	local reset_list = {}
    	for i,val in pairs(mob_list) do
    		local obj = GameObj(tonumber(val))
    		if(obj ~= nil) then
    			table.insert(reset_list,obj:GetCreationTemplateId())
    			obj:Destroy()
    		end
    	end
    	this:SetObjVar("reset_list",reset_list)
    	this:SetObjVar("sleeping",true)
    	UpdateToolTip(this,false)
    	else
    		print("Something is trying to put me to sleep, but im already sleeping - [SpawnerID#"..this.Id.."]")
    	end
    end)

----------------------------------------------------------
--EVENTTYPE - Message - "Wakeup"
----------------------------------------------------------
--this handles the wakeup message
RegisterEventHandler(EventType.Message, "Wakeup", 
    function()
    	if(this:GetObjVar("sequential")==true) then
    		return
    	end    	
    	if(this:GetObjVar("sleeping")) then
	    	local reset_list = this:GetObjVar("reset_list")
	    	local min_range = this:GetObjVar("min_range")
	    	local max_range = this:GetObjVar("max_range")
	    	local templates = this:GetObjVar("templates")
	    	this:SetObjVar("mob_list",{})
	    	local spawn_mobile = ""
			local spawn_template = templates[this:GetObjVar("current_sequence")]
			local data = StringSplit(spawn_template,COMMAND_KEY)
			if (spawn_template == nil or data[1] == nil) then return end
			local spawn_mobile = data[1]

	    	if(reset_list ~= nil) then
	    		for i,template in pairs(reset_list) do
	    			CreateObj(data[1],this:GetLoc():Project(math.random(0,360), math.random(min_range,max_range)),"mobileSpawned",1,spawn_template)
	    		end
	    	end
	    	this:SetObjVar("count_current",#this:GetObjVar("mob_list"))
    		this:SetObjVar("sleeping",false)
    		UpdateToolTip(this,false)
    	end
    end)

----------------------------------------------------------
--EVENTTYPE - Message - "Respawn"
----------------------------------------------------------
--this handles the respawn message
RegisterEventHandler(EventType.Message, "Respawn", 
    function()
    	if not(this:GetObjVar("sleeping")) then
    		if(this:GetObjVar("active")) then
	    		local mob_list = this:GetObjVar("mob_list")
	    		local templates = this:GetObjVar("templates")
	    		local min_range = this:GetObjVar("min_range")
	    		local max_range = this:GetObjVar("max_range")
				local spawn_mobile = ""
				--current_sequence should never be 0 to start
				if(this:GetObjVar("current_sequence") == 0) then
					this:SetObjVar("current_sequence",1)
				end				
				local spawn_template = templates[this:GetObjVar("current_sequence")]
				local data = StringSplit(spawn_template,COMMAND_KEY)
				local spawn_mobile = data[1]
	    		for i = #mob_list, 1, -1 do
	    			local object = GameObj(mob_list[i])
					if(object:HasObjVar("parentSpawner") and object:HasModule("spawner")) then
						local spwn_mob_list = object:GetObjVar("mob_list")
						if(#spwn_mob_list > 0) then
							for d=#spwn_mob_list, 1, -1 do
								local spwn_object = GameObj(tonumber(spwn_mob_list[d]))
								spwn_object:Destroy()
							end
						end
					end
					object:Destroy()	    			
	    		end
				this:SetObjVar("count_current",0)
	    		this:SetObjVar("mob_list",{})    		
	    		for i = 1, this:GetObjVar("count_max") do
	    			if(this:GetObjVar("sequential") == true) then
	    				CreateObj(spawn_mobile,this:GetLoc():Project(math.random(0,360), math.random(min_range,max_range)),"mobileSpawned",1,spawn_template)
	    			else
	    				CreateObj(StringSplit(templates[math.random(1,#templates)],COMMAND_KEY)[1],this:GetLoc():Project(math.random(0,360), math.random(min_range,max_range)),"mobileSpawned",1,spawn_template)
	    			end
	    		end
    		end
    		if(this:HasTimer("spawnerTimer")) then
    			this:RemoveTimer("spawnerTimer")
    			this:ScheduleTimerDelay(TimeSpan.FromMilliseconds(GetRespawnDelay()), "spawnerTimer")
    		end
    	end
    end)

----------------------------------------------------------
--EVENTTYPE - Message - "Reset"
----------------------------------------------------------
--when the /resetworld command is fired in our scriptcommands.lua
--we will call this to correct the spawner
RegisterEventHandler(EventType.Message, "Reset", 
    function()
    	if(this:GetObjVar("sequential")==true) then
    		this:SetObjVar("mob_list",{})
    		this:DelObjVar("PhaseSpawned")
    		SequentialSpawnObject()
    	else
    		this:SendMessage("Respawn")
    	end
    end)

----------------------------------------------------------
--EVENTTYPE - LoadedFromBackup - ""
----------------------------------------------------------
RegisterEventHandler(EventType.LoadedFromBackup,"",SpawnerUpdate)

----------------------------------------------------------
--EVENTTYPE - Message - "activate"
----------------------------------------------------------
--this message is sent from the spawn_editor_menu
RegisterEventHandler(EventType.Message, "activate", 
    function()
    	if not(this:GetObjVar("active")) then
    		this:SetObjVar("active",true)
			this:ScheduleTimerDelay(TimeSpan.FromMilliseconds(GetRespawnDelay()), "spawnerTimer")
			this:SetObjVar("View_NextTrigTime",os.time()+this:GetObjVar("View_ReTriggerDelay"))
    	end
    end)

----------------------------------------------------------
--EVENTTYPE - Message - "deactivate"
----------------------------------------------------------
--this message is sent from the spawn_editor_menu
RegisterEventHandler(EventType.Message, "deactivate", 
    function()
	    if (this:GetObjVar("active")) then
    		this:SetObjVar("active",false)
    		this:SetObjVar("count_current",0)
    		this:SetObjVar("kill_count",0)
    		this:SetObjVar("current_sequence",0)
    		this:SetObjVar("View_Triggered",false)
    		this:SetObjVar("View_TriggeredMob",0)
    		ShutDown()
    		CallFunctionDelayed(TimeSpan.FromMilliseconds(500),function ( ... )
			if(HasTooltipEntry(this,"is_being_edited")) then
				UpdateToolTip(this,true)
			else
				UpdateToolTip(this,false)
			end
    		end)    		
    	end    	
    end)

----------------------------------------------------------
--EVENTTYPE - CreatedObject - "mobileSpawned"
----------------------------------------------------------
--this even is fired when we successfuly create the spawned mobile
--it will attach a module and variable to the created mobile
RegisterEventHandler(EventType.CreatedObject, "mobileSpawned", 
	function(success, objref, index,spawn_template)
		if( success) then
			local current_count = this:GetObjVar("count_current")
			objref:SetObjVar("parentSpawner",this.Id)
			objref:AddModule("spawner_on_object_destroy")
			if not(objref:IsMobile() and objref:HasObjVar("ResourceSourceId")) then
				SetItemTooltip(objref)
			end

			current_count = current_count + 1
			this:SetObjVar("count_current",current_count)

			local mob_list = this:GetObjVar("mob_list")
			table.insert(mob_list,objref.Id)
			this:SetObjVar("mob_list",mob_list)

			HandleChanges(objref,spawn_template)
		end
		if(HasTooltipEntry(this,"is_being_edited")) then
			UpdateToolTip(this,true)
		else
			UpdateToolTip(this,false)
		end
		if(objref:HasModule("container")) then
			CreateLootOnMobile(objref)
		end
	end)

----------------------------------------------------------
--EVENTTYPE - CreatedObject - "createdStackableLootOnMobile"
----------------------------------------------------------
--this takes care of items spawned with quantity
RegisterEventHandler(EventType.CreatedObject, "createdStackableLootOnMobile", 
	function(success,objRef,quantity)
		if( success == true ) then
			if( quantity ~= nil and quantity > 0 ) then
				if(objRef:GetCreationTemplateId() == "coin_purse") then
					local max_coins = quantity
					local min_coins = math.floor(quantity * 0.40)
					print("random "..min_coins.."-"..max_coins)
					if (max_coins <= 0) then max_coins = 0 end
					if (min_coins <= 0) then min_coins = 1 end
					RequestSetStackCount(objRef,math.random(min_coins,max_coins))
				else		
					RequestSetStackCount(objRef,quantity)
				end
			end
		end
	end)

----------------------------------------------------------
--EVENTTYPE - CreatedObject - "createdLootOnMobile"
----------------------------------------------------------
--this takes care of items spawned without quantity
RegisterEventHandler(EventType.CreatedObject, "createdLootOnMobile", 
	function(success, objref,quantity,params)
		if(type(quantity) == "number") then
			if(success and quantity > 1) then
				RequestSetStack(objRef,quantity)
			end
		end
		if(params.Rank ~= nil) then
			if(objref:HasObjVar("WeaponType")) then
				LootTable.Functions.ApplyWeaponProperties(objref,params.Rank)
			elseif(objref:HasObjVar("ArmorType")) then
				LootTable.Functions.ApplyArmorProperties(objref,params.Rank)
			elseif(objref:HasObjVar("ShieldType")) then
				LootTable.Functions.ApplyOffHandProperties(objref,params.Rank)
			end
		else
			SetItemTooltip(objref)
		end			
	end)

----------------------------------------------------------
--EVENTTYPE - DynamicWindowResponse - EditHueDialog
----------------------------------------------------------
--you can find the dialog window information in gizmos_spawner_hue_picker.lua
--the message is sent from the hue picker set button
RegisterEventHandler(EventType.DynamicWindowResponse,"EditHueDialog", 
	function (user,buttonId)
		if(buttonId ~= nil) then
			local items = StringSplit(buttonId,":")
			local action = items[1]
			local param = items[2]
			if(param ~= nil) then
				this:SetHue(22)
				this:SetColor(param)
				user:SystemMessage("you have set the hue of the spawner")
			end
		end
	end)

----------------------------------------------------------
--EVENTTYPE - DynamicWindowResponse - EditLootTemplate
----------------------------------------------------------
--you can find the dialog window information in gizmos_spawner_edit_window.lua
--the message is sent from the edit window dialog response function
RegisterEventHandler(EventType.DynamicWindowResponse,"EditLootTemplate", 
	function (user,buttonId,textField)
		if(buttonId ~= nil) then
			local items = StringSplit(buttonId,":")
			local action = items[1]
			local param = items[2]
			if(action == "changeTemplateData") then
				local lootTemplate = {textField.edit_template_field,textField.edit_chance_field,textField.edit_quanitity_field}
				local templates = this:GetObjVar("loot")
				table.remove(templates,tonumber(param))
				table.insert(templates,tonumber(param),lootTemplate)
				this:SetObjVar("loot",templates)
				SimpleSpawnerEditWindow(user,this,"Loot")
			else
				user:SystemMessage("you decide not to change the loot template")
				--SimpleSpawnerEditWindow(user,this,"Loot")
			end
		end
	end)

local SavedIndex = 0
local SavedUser = nil

----------------------------------------------------------
--EVENTTYPE - DynamicWindowResponse - EditTemplateEntry
----------------------------------------------------------
--you can find the dialog window information in gizmos_spawner_edit_window.lua
--the message is sent from the edit window dialog response function
RegisterEventHandler(EventType.DynamicWindowResponse,"EditTemplateEntry", 
	function (user,buttonId,textField)
		if(buttonId ~= nil) then
			local items = StringSplit(buttonId,":")
			local action = items[1]
			local param = items[2]
			if(action == "changeTemplateData") then
				local templates = this:GetObjVar("templates")
				--local unformattedtext = string.gsub(textField.edit_template_field,"\n","")
				table.remove(templates,tonumber(param))
				table.insert(templates,tonumber(param),textField.edit_template_field)
				this:SetObjVar("templates",templates)
				SimpleSpawnerEditWindow(user,this,"Spawns")
			else
				user:SystemMessage("you decide not to change the loot template")
				--SimpleSpawnerEditWindow(user,this,"Spawns")
			end
			if(action == "getobject") then
        		user:RequestClientTargetAnyObj(this,"GetObjectInfo")
        		user:SystemMessage("select the object you want to get object data from")				
        		SavedIndex = param
        		SavedUser = user
			end
			if(action == "getlocation") then
        		user:RequestClientTargetAnyObj(this,"GetLocationInfo")
        		user:SystemMessage("select the object you want to get object data from")				
        		SavedIndex = param
        		SavedUser = user
			end
			if(action == "gethue") then
        		user:RequestClientTargetAnyObj(this,"GetHueInfo")
        		user:SystemMessage("select the object you want to get object data from")				
        		SavedIndex = param
        		SavedUser = user
			end	
			if(action == "getrotation") then
        		user:RequestClientTargetAnyObj(this,"GetRotationInfo")
        		user:SystemMessage("select the object you want to get object data from")				
        		SavedIndex = param
        		SavedUser = user
			end	
			if(action == "gettemplate") then
        		user:RequestClientTargetAnyObj(this,"GetTemplateInfo")
        		user:SystemMessage("select the object you want to get object data from")				
        		SavedIndex = param
        		SavedUser = user
			end	
			if(action == "fill_object") then
    			local templates = this:GetObjVar("templates")
    			local template_data = templates[tonumber(param)]
    			table.remove(templates,tonumber(param))
				table.insert(templates,tonumber(param),template_data.."OBJECT")
				this:SetObjVar("templates",templates)
				ShowEditorWindow(user,this,template_data.."OBJECT",nil,param)				
			end
			if(action == "fill_self") then
    			local templates = this:GetObjVar("templates")
    			local template_data = templates[tonumber(param)]
    			table.remove(templates,tonumber(param))
				table.insert(templates,tonumber(param),template_data.."SELF")
				this:SetObjVar("templates",templates)				
				ShowEditorWindow(user,this,template_data.."SELF",nil,param)
			end
			if(action == "fill_at_symbol") then
    			local templates = this:GetObjVar("templates")
    			local template_data = templates[tonumber(param)]
    			table.remove(templates,tonumber(param))
				table.insert(templates,tonumber(param),template_data.."@")
				this:SetObjVar("templates",templates)				
				ShowEditorWindow(user,this,template_data.."@",nil,param)
			end
			if(action == "fill_objectlist") then
    			local templates = this:GetObjVar("templates")
    			local template_data = templates[tonumber(param)]
    			table.remove(templates,tonumber(param))
				table.insert(templates,tonumber(param),template_data.."OBJECTLIST")
				this:SetObjVar("templates",templates)				
				ShowEditorWindow(user,this,template_data.."OBJECTLIST",nil,param)
			end																		
		end
	end)

RegisterEventHandler(EventType.ClientTargetAnyObjResponse, "GetObjectInfo", 
    function(target)
    	if(target ~= nil) then
    		local templates = this:GetObjVar("templates")
    		local template_data = templates[tonumber(SavedIndex)]
    		local id = "SELF"
    		if(target.Id ~= this.Id) then id = target.Id end
    		ShowEditorWindow(SavedUser,this,template_data,"OBJECT@"..id,SavedIndex)
    	end
    end)

RegisterEventHandler(EventType.ClientTargetAnyObjResponse, "GetLocationInfo", 
    function(target)
    	if(target ~= nil) then
    		local templates = this:GetObjVar("templates")
    		local template_data = templates[tonumber(SavedIndex)]
    		local id = "SELF"
    		if(target.Id ~= this.Id) then id = target.Id end
    		if(target:GetLoc() ~= nil) then
    			local location = target:GetLoc()
	    		ShowEditorWindow(SavedUser,this,template_data,location.X..","..location.Y..","..location.Z,SavedIndex)
    		end
    	end
    end)

RegisterEventHandler(EventType.ClientTargetAnyObjResponse, "GetHueInfo", 
    function(target)
    	if(target ~= nil) then
    		local templates = this:GetObjVar("templates")
    		local template_data = templates[tonumber(SavedIndex)]
    		local id = "SELF"
    		if(target.Id ~= this.Id) then id = target.Id end
    		if(target:GetColor() ~= nil) then
	    		ShowEditorWindow(SavedUser,this,template_data,target:GetColor(),SavedIndex)
    		end
    	end
    end)

RegisterEventHandler(EventType.ClientTargetAnyObjResponse, "GetRotationInfo", 
    function(target)
    	if(target ~= nil) then
    		local templates = this:GetObjVar("templates")
    		local template_data = templates[tonumber(SavedIndex)]
    		local id = "SELF"
    		if(target.Id ~= this.Id) then id = target.Id end
    		if(target:GetLoc() ~= nil) then
    			local rotation = target:GetRotation()
	    		ShowEditorWindow(SavedUser,this,template_data,rotation.X..","..rotation.Y..","..rotation.Z,SavedIndex)
    		end
    	end
    end)

RegisterEventHandler(EventType.ClientTargetAnyObjResponse, "GetTemplateInfo", 
    function(target)
    	if(target ~= nil) then
    		local templates = this:GetObjVar("templates")
    		local template_data = templates[tonumber(SavedIndex)]
    		local id = "SELF"
    		if(target.Id ~= this.Id) then id = target.Id end
    		if(target:GetColor() ~= nil) then
	    		ShowEditorWindow(SavedUser,this,template_data,target:GetCreationTemplateId(),SavedIndex)
    		end
    	end
    end)

----------------------------------------------------------
--EVENTTYPE - DynamicWindowResponse - EditSpawnerDialog
----------------------------------------------------------
--you can find the dialog window information in gizmos_spawner_edit_window.lua
--the message is sent from the edit window dialog response function
RegisterEventHandler(EventType.DynamicWindowResponse,"EditSpawnerDialog", 
	function (user,buttonId,textField)
		if(buttonId ~= nil) then
			local items = StringSplit(buttonId,":")
			local action = items[1]
			local param = items[2]

			--the below code is what takes care of the tabbing system
			if(action == "Spawns") then SimpleSpawnerEditWindow(user,this,"Spawns") end
			if(action == "Config") then SimpleSpawnerEditWindow(user,this,"Config") end
			if(action == "Search") then SimpleSpawnerEditWindow(user,this,"Search") end
			if(action == "Loot") then SimpleSpawnerEditWindow(user,this,"Loot") end
			if(action == "Active") then SimpleSpawnerEditWindow(user,this,"Active") end

			if(action == "PushEntryUp") then
				local spawns = this:GetObjVar("templates")
				local entry = spawns[tonumber(param)]
				if(entry ~= nil) then
					table.remove(spawns,tonumber(param))
					table.insert(spawns,(tonumber(param)-1),entry)
					this:SetObjVar("templates",spawns)
					SimpleSpawnerEditWindow(user,this,"Spawns")
				end
			end

			if(action == "PushEntryDown") then
				local spawns = this:GetObjVar("templates")
				local entry = spawns[tonumber(param)]
				if(entry ~= nil) then
					table.remove(spawns,tonumber(param))
					table.insert(spawns,(tonumber(param)+1),entry)
					this:SetObjVar("templates",spawns)
					SimpleSpawnerEditWindow(user,this,"Spawns")
				end
			end

			if(action == "EditLootTemplate") then
				local window = DynamicWindow("EditLootTemplate","[BBBB44]Editing Loot Info",500,120,-300,-300,"Default","Center")
				local templates = this:GetObjVar("loot")

				window:AddLabel(13,10,"Template",90,20,20,"left",false,false,"PermianSlabSerif_16_Dynamic")
				window:AddTextField(85,8,390,20,"edit_template_field",templates[tonumber(param)][1])
				window:AddLabel(13,30,"Chance",90,20,20,"left",false,false,"PermianSlabSerif_16_Dynamic")
				window:AddTextField(85,30,150,20,"edit_chance_field",templates[tonumber(param)][2])
				window:AddLabel(240,30,"Quntity",90,20,20,"left",false,false,"PermianSlabSerif_16_Dynamic")
				window:AddTextField(305,30,170,20,"edit_quanitity_field",templates[tonumber(param)][3])
				window:AddButton(10,55,"changeTemplateData:"..param,"Save",230,28,"[FF7700]Save\n[FFFFBB]save the loot template data",nil,true,"SimpleButton")
				window:AddButton(240,55,"cancel","Cancel",230,28,"[FF7700]Cancel\n[FFFFBB]cancel editing the loot template data",nil,true,"SimpleButton")

				user:OpenDynamicWindow(window,this)

			end 
			if(action == "EditTemplateEntry") then
				local templates = this:GetObjVar("templates")
				ShowEditorWindow(user,this,templates[tonumber(param)],"",param)
			end			
			if(action == "Deactivate") then	
				this:SendMessage("deactivate")
				if(HasTooltipEntry(this,"is_being_edited")) then
					UpdateToolTip(this,true)
				else
					UpdateToolTip(this,false)
				end
				SimpleSpawnerEditWindow(user,this,"Config")
				user:SystemMessage("spawner deactivated and is now offline")
			end
			if(action == "Activate") then
				this:SendMessage("activate")
				if(HasTooltipEntry(this,"is_being_edited")) then
					UpdateToolTip(this,true)
				else
					UpdateToolTip(this,false)
				end
				SimpleSpawnerEditWindow(user,this,"Config")
				user:SystemMessage("spawner activated and is now online")
			end
			if(action == "Sleep") then
				this:SendMessage("Sleep")
			end
			if(action == "Wakeup") then
				this:SendMessage("Wakeup")
			end
			if(action == "Respawn") then
				this:SendMessage("Respawn")
			end						
			if(action == "SequentialOff") then
				this:SetObjVar("sequential",false)
				UpdateToolTip(this, true)
				SimpleSpawnerEditWindow(user,this,"Config")
				this:SendMessage("Respawn")
			end 
			if(action == "SequentialOn") then
				this:SetObjVar("sequential",true)
				UpdateToolTip(this, true)
				SimpleSpawnerEditWindow(user,this,"Config")
				this:SendMessage("Respawn")
			end			
			--this fires the script xml file function
			if(action == "CreateIt") then
				if(user:GetObjVar("ModName") ~= nil) then
					local file = io.open("mods/".. user:GetObjVar("ModName") .."/templates/spawners/junka.lua","w")
					if(file ~= nil) then
						local directory = "mods/".. user:GetObjVar("ModName") .."/templates/spawners/"
						local name, color = StripColorFromString(this:GetName())
						name = name:lower()
						name = string.gsub(name,"- ","")
						name = string.gsub(name,"-","")
						name = string.gsub(name,"'","")
						local filename = string.gsub(name," ","_")
						if(filename=="spawner") then
							user:SystemMessage("[FF7700]Name Error")
							user:SystemMessage("[FFDD00]you can not save as gizmos_simple_spawner...rename your spawner in the config menu.")
							return
						end
						local fullpath = directory .. "spawner_" .. filename .. ".xml"
						CreateTemplate(fullpath,user)
						--CallFunctionDelayed(TimeSpan.FromMilliseconds(1000),function ( ... ) ReloadTemplates() end)
					else
						user:SystemMessage("[FF7700]-------------------------------------------------")
						user:SystemMessage("[22FF22]                  -[SETUP REQUIRED]-")
						user:SystemMessage("[FF7700]-------------------------------------------------")
						user:SystemMessage("[22FF22]Simply Do The Following")
						user:SystemMessage("[FFFFBB]I see you completed step 1 by creating your ModName variable. One more step and we will be ready. Create a folder named [FF7700]'spawners' [FFFFBB] in your mods/"..user:GetObjVar("ModName").."/templates directory")
					end
				else
					user:SystemMessage("[FF7700]-------------------------------------------------")
					user:SystemMessage("[22FF22]                  -[SETUP REQUIRED]-")
					user:SystemMessage("[FF7700]-------------------------------------------------")
					user:SystemMessage("[22FF22]Simply Do The Following")
					user:SystemMessage("[FFFFBB]Add a string variable named [FF7700]'ModName'[FFFFBB] to your character. Name this variable the same name as your mod name, remember folders are case sensitive. This is so you can save these templates directly to your mod folder. These templates will be placed in the following directory [FF7700]'yourmod/templates/merchants'")
				end
			end
			--this is fired when you click the insert button in the Spawns tab
			if(action == "AddMobileToList") then
				if(textField.template_reference ~= nil) then
					local list = this:GetObjVar("templates")
					table.insert(list,textField.template_reference)
					this:SetObjVar("templates",list)
					SimpleSpawnerEditWindow(user,this)
					user:SystemMessage("you have successfully added the mob to the spawn table")
				end
			end
			--this is fired when you click the [X] in the Spawns tab
			if(action == "DeleteMobFromList") then
				local list = this:GetObjVar("templates")
				table.remove(list,tonumber(param))
				this:SetObjVar("templates",list)
				SimpleSpawnerEditWindow(user,this)
				user:SystemMessage("you have successfully removed the mob from the spawn table")
			end
			--this is fired when you click the edit option in the
			--config menu. it will gather the information from the
			--text field and enter it into the spawners objvar.
			if(action == "EditConfig") then
				if(param == "min_delay") then this:SetObjVar("min_delay",tonumber(textField.prop_value_min_delay)) end
				if(param == "max_delay") then this:SetObjVar("max_delay",tonumber(textField.prop_value_max_delay)) end
				if(param == "min_range") then this:SetObjVar("min_range",tonumber(textField.prop_value_min_range)) end
				if(param == "max_range") then this:SetObjVar("max_range",tonumber(textField.prop_value_max_range)) end
				if(param == "count_max") then this:SetObjVar("count_max",tonumber(textField.prop_value_count_max)) end
				if(param == "spawner_tag") then this:SetObjVar("spawner_tag",textField.prop_value_spawner_tag) end

				UpdateConfig()
				UpdateToolTip(this,true)
				SimpleSpawnerEditWindow(user,this,"Config")
				user:SystemMessage("configuration sucessfully added to the spawner")
			end
			--this will delete a loot template from the loot table list
			if(action == "DeleteLootTemplate") then		
				local list = this:GetObjVar("loot")
				table.remove(list,tonumber(param))
				this:SetObjVar("loot",list)
				SimpleSpawnerEditWindow(user,this,"Loot")
				user:SystemMessage("item value successfully removed from the spawner")		
			end
			--this is fired when you click the insert button in the Loot tab
			if(action == "AddItemToList") then
				if(textField.item_reference ~= "") then
					local list = this:GetObjVar("loot")
					local data = StringSplit(textField.item_reference,":")
					local loot_table = {textField.loot_template,textField.loot_chance,textField.loot_quantity}
					local doinsert = true
					if(textField.loot_template == nil or textField.loot_chance == nil or textField.loot_quantity == nil) then
						doinsert = false
						user:SystemMessage("you entered bad data, try again")
					end
					if(doinsert) then
						table.insert(list,loot_table)
						this:SetObjVar("loot",list)
						user:SystemMessage("item values successfully added to the spawner")
					end
					SimpleSpawnerEditWindow(user,this,"Loot")
				end
			end
			--this is fired when you click home button on the active tab
			if(action == "SendHome") then
				local mob_list = this:GetObjVar("mob_list")						
				local mobile = GameObj(mob_list[tonumber(param)])
				mobile:SetWorldPosition(this:GetLoc())
				user:SystemMessage(mobile:GetName().." has be sent back home")
			end	
			--this is fired when you click the change name edit button in 
			--the spawner configu menu
			if(action == "ChangeName") then
				print(textField.spawner_name_value:lower())
				local name = textField.spawner_name_value:lower()
				name = string.gsub(name,"- ","")
				name = string.gsub(name,"-","")
				name = string.gsub(name,"'","")
				local filename = string.gsub(name," ","_")
				if(DoesTemplateExist(filename) == true) then
					user:SystemMessage("[FF6666]-=| WARNING!!! WARNING!!! |=-")
					user:SystemMessage("[FFFF00][FFFFFF][spawner_"..filename.."][FFFF00]\nis a template that alread exist, you will be over writing a previous version of this spawner. If you are unsure if you should save over this, please rename the spawner or contact a server admin.")
				end
				this:SetName("[CCCC00]"..textField.spawner_name_value)
				UpdateToolTip(this,true)
				user:SystemMessage("you have successfully changed the name of this spawner")
				
				SimpleSpawnerEditWindow(user,this,"Config")
			end

			--------------------------------------
			-- SEARCH FEATURE BUTTON CLICKS 
			--------------------------------------
			--this is fired when you click Name button on the search tab
			if(action == "SearchName") then
				local queryString = textField.query_value
				if(queryString ~= nil) then
					SimpleSpawnerEditWindow(user,this,"Search","Name:"..queryString)
					user:SystemMessage("you are querying for spawners with a name including ['".. queryString .."']")
				end
			end
			--this is fired when you click template button on the search tab
			if(action == "SearchTemplate") then
				local queryString = textField.query_value
				if(queryString ~= nil) then
					SimpleSpawnerEditWindow(user,this,"Search","Template:"..queryString)
					user:SystemMessage("you are querying for spawners with a mobile template including ['".. queryString .."']")
				end
			end
			--this is fired when you click objectid button on the search tab
			if(action == "SearchObjectId") then
				local queryString = textField.query_value
				if(queryString ~= nil) then
					SimpleSpawnerEditWindow(user,this,"Search","Id:"..queryString)
					user:SystemMessage("you are querying for spawners with a object id including ['".. queryString .."']")
				end
			end
			--this is fired when you click objectid button on the search tab
			if(action == "SearchLootTemp") then
				local queryString = textField.query_value
				if(queryString ~= nil) then
					SimpleSpawnerEditWindow(user,this,"Search","LootTemplate:"..queryString)
					user:SystemMessage("you are querying for spawners with a loot item including ['".. queryString .."']")
				end
			end
			--this is fired when you click objvar button on the search tab
			if(action == "SearchObjVar") then
				local queryString = textField.query_value
				if(queryString ~= nil) then
					SimpleSpawnerEditWindow(user,this,"Search","ObjVar:"..queryString)
					user:SystemMessage("you are querying for spawners with a objvar including ['".. queryString .."']")
				end
			end
			--this is fired when you click tag button on the search tab
			if(action == "SearchTag") then
				local queryString = textField.query_value
				if(queryString ~= nil) then
					SimpleSpawnerEditWindow(user,this,"Search","Tag:"..queryString)
					user:SystemMessage("you are querying for spawners with a tag including ['".. queryString .."']")
				end
			end
			----------------------------------------
			-- SEARCH FEATURE QUERY BUTTON CLICKS 
			----------------------------------------	
		
			--this is fired when you click the Jump To button in the
			--search tab query field list
			if(action == "JumpToSearch") then
				local spawner = GameObj(tonumber(param))
				user:PlayEffect("TeleportToEffect")
				user:SetWorldPosition(spawner:GetLoc())
				user:PlayEffect("TeleportFromEffect")
				user:SystemMessage("you have been moved to the spawner")
			end	
			--this is fired when you click the Edit button in the
			--search tab query field list
			if(action == "EditSearch") then
				local spawner = GameObj(tonumber(param))
				SimpleSpawnerEditWindow(user,spawner)
				if(HasTooltipEntry("is_being_edited")) then
					RemoveTooltipEntry(this,"is_being_edited")
				end
				UpdateToolTip(spawner,true)
				user:SystemMessage("you are now editing the spawner '"..string.sub(spawner:GetName(),9).."'")
			end
			--this is fired when you click the Turn On/Turn Off button in the
			--search tab query field list
			if(action == "ActivateSearch") then
				local data = StringSplit(param,"|")
				local spawner = GameObj(tonumber(data[1]))
				if(spawner:GetObjVar("active")) then
					spawner:SetObjVar("active",false)
					user:SystemMessage("you have deactivated the spawner '"..string.sub(spawner:GetName(),9).."'")
				else
					spawner:SetObjVar("active",true)
					user:SystemMessage("you have activated the spawner '"..string.sub(spawner:GetName(),9).."'")
				end
				SimpleSpawnerEditWindow(user,this,"Search","Name:"..data[2])
				UpdateToolTip(spawner,true)
			end
		end
		if(buttonId == "") then
			UpdateToolTip(this,false)
		end
	end)

----------------------------------------------------------
--EVENTTYPE - Timer - "spawnTimer"
----------------------------------------------------------
--this is the defibrillator of the system. should
--be left alone unless you know what your doing
RegisterEventHandler(EventType.Timer, "spawnerTimer", 
	function()
		UpdateToolTip(this,false)
		if(this:GetObjVar("active") == true) then
			if not(this:GetObjVar("sequential")) then
				if(this:GetObjVar("count_current") < this:GetObjVar("count_max")) then
					SpawnObject()
				end
			else
				if not(this:HasObjVar("PhaseSpawned")) then
					SequentialSpawnObject()
				end
			end
		end
		if(HasTooltipEntry(this,"is_being_edited")) then
			UpdateToolTip(this,true)
		else
			UpdateToolTip(this,false)
		end
		if(this:GetObjVar("active") == true) then
			this:ScheduleTimerDelay(TimeSpan.FromMilliseconds(GetRespawnDelay()), "spawnerTimer")
		end
		if(this:HasObjVar("View_TriggerActive")) then
			UpdateToolTip(this,false)
		end
	end)

